# Latency Issues detection & AI recommendation

## Overview

This project contains all instructions and programs for Latence issues recommendation using OpenAI assistant api.

The created container in this project includes :

_- An api REST using POST and GET method for request/response to the assistant program_

_- Storage for script and useful file for analyzer dashboard_

The container could be hosted in the same server as the analyzer. It may be hosted away but a pre-installed Docker desktop is required.

## Description & architecture

* _Description_

A request for gathering data is sent to the influxdb, selecting the bucket where latency measurements are written.

These data are then aggregated and filtered to keep only the measurement where issues appears in each protocol.

The result is shown on the dashboard and formatted according to the level of the issues (a warning or a real issue which is needed to be fixed).

An buttom for generating an AI recommendation from openAI platform can be seen in this dashboard.

* _Architecture_

The following figure summarize the system :

![Architecture](src/include/archi.jpg)

## Initial Configuration

For it's deployment, following are the informations we need :

* An api key generated from the OpenAI platform ([here](https://platform.openai.com/api-keys))

* For the OpenAI assistant :
  * An instruction for OpenAI model on how to process the recommendation and display the results ([a template here](files/instructions.txt)) and all useful file for the assistant (should be stored in a folder named "docs")
  * or an assistant ID if the assistant is already defined

* A public IP address or dns name of the server where the container will be deployed.

* Opening and exposed ports from 5000 to 5004 for data exchange

## Assistant creation

### Assistant program

The program in [assistant.py](src/assistant.py) can create an assistant for OpenAI assistant API.

You just need to provide some information such as :

* an API key generated from the OpenAI platform ([here](https://platform.openai.com/api-keys))
* an instructions file for assistant api ([template here](files/instructions.txt))
* a location where useful file are stored

### Usage for assistant

To get help for assistant creation, just run

```
python3 assistant.py --help
```

it will shows help on how to provide all arguments. Following is an example of usage :

```
python3 assistant.py --name "Assistant Recommendation" --api_key=Add-here-the-api-key-generated-from-OpenAI-platform --instructions_file=../files/instruction.txt --model=gpt-4-1106-preview  --tools=retrieval,code_interpreter  --files=../docs
```

## Openai main program

The main program here read all required input variable such as port number, id of assistant to use and OpenAI api key fro the host environment variable:

* `OPENAI_API_KEY` for api key
* `OPENAI_ASSISTANT_ID` for the assistant to use
* `ASSISTANT_PORT` for port number
* `FILE_PORT` use port for shared file (csv/js)

It listen to the request sent accross the api endpoint (`/api/recommendation/get` or `/api/recommendation/post`) according to used method.

When this api is solicited by a client, the program create a new thread for the request, using the assistant Id as well as the api key, send the request to the OpenAI platform and wait for the response.

While the platform finish processing the request and send a result, this result is directly transmitted to the client through the Api.

### a. Docker building

For building a docker image from this project, the command is :

`docker build -t <imagename> .`

example : `docker build -t openaiserver .`

### b. Launching the container

two options are possible for launching a container from the image:

_**Using host environment variable**_

This option use the host environment variable which contains all input such as the api key, assistant id, api port ...

Here is the command :

```
docker run -p 0.0.0.0:$ASSISTANT_PORT:$ASSISTANT_PORT -p 0.0.0.0:$FILE_PORT:$FILE_PORT --name openai -e OPENAI_API_KEY -e OPENAI_ASSISTANT_ID -d openaiserver
```

_**Using specific variable**_

In this case, you need to provide all required variable (OPENAI_API_KEY, OPEAI_ASSISTANT_ID, ASSISTANT_PORT, FILE_PORT)

Here i the command :

```
docker run -p 0.0.0.0:<ASSISTANT_PORT>:<ASSISTANT_PORT> -p 0.0.0.0:<FILE_PORT>:<FILE_PORT> --name openai -e OPENAI_API_KEY=<OPENAI_API_KEY> -e OPENAI_ASSISTANT_ID=<OPENAI_ASSISTANT_ID> -d openaiserver
```

example :

```
docker run -p 0.0.0.0:5002:5002 -p 0.0.0.0:5004:5004 --name openai -e OPENAI_API_KEY=sk-123324abcd***** -e OPENAI_ASSISTANT_ID=asst_sabendi***** -d openaiserver
```

## Launch everyting at once

For launching everything at once, the configuration file in [ltiapiopenai.conf](config/ltiapiopenai.conf) should contains all variables.

Here is the content to fill :

```
API_KEY=<put here the api key for OpenAI platform>
ASS_ID=<put here the id of the assistant>
ASS_PORT=<put here the port to use (5002)>
IP_ADDRESS=<Put here the public address of the server>
IMAGE_NAME=<The name for docker image (openaiserver)>
CONTAINER_NAME=<the name for container (openai)>
F_PORT=<port to use for sharing port (5004)>
```

When the ltiapiopenai.conf is ready, use the [deploy.sh](src/deploy.sh) to deploy the container.

Here is the command :

```
cd src
sh deploy.sh
```

This program will ask you if you want a new assistant or not.

If the response is "y" (yes), the program will launch the assistant creation and create a new assistant based on the provided api key and the [instruction file](files/instructions.txt)

If not, it will use the assistant according to the assistant ID in the configuration file.

The deploy script will generate as well a Json file for the dashboard and it will be located in [dashboard/](dashboard/issue_dashboard.json)
