# This contains functions for new assistant creation and add useful files
import os
import argparse
import glob
from openai import OpenAI
import numpy as np

api_key = os.environ['OPENAI_API_KEY']
client = OpenAI(api_key=f'{api_key}')

# print(instructions)

def createAssistant(args: argparse.Namespace) -> None:
    """Run this program to create a new assistant for openAI assistant api.
    Args:
        args (argparse.Namespace): Args that contains all the arguments for the server to function.
        --name : define the name for the assistant. This one is required.
        --api_key : define the OpenAI api key to use (Default value is from the environment variable OPENAI_API_KEY)
        --instruction_file : precise the exact path of a text file which contains the instructions for the assistant
        --model : The name of model to use (the default value isgpt-4-1106-preview)
        --tools : Type of tools to use (code_interpreter/retrieval)
        --files : Folder path where all files (pdf,json) are located (default location ../docs)
    """
    theTools = ""
    argTools = args.tools.split(",")
    if len(argTools) > 1 :
        theTools = [{"type":argTools[0]},{"type":argTools[1]}]
    else:
        theTools = [{"type":argTools[0]}]
    instruction_file = open(args.instructions_file, "r")
    instructions = instruction_file.read()
    assistant = client.beta.assistants.create(
        name = args.name,
        instructions = instructions,
        # tools = [{"type":args.tools}],
        tools = theTools,
        model = args.model,
        #file_ids = ""
    )
    return assistant.id

# List all files in a folder using glob library 
def fileList(folderPath):
    pathList = []
    if glob.glob(f"{folderPath}/*.pdf") != []:
        pathList.append(glob.glob(f"{folderPath}/*.pdf"))
    if glob.glob(f"{folderPath}/*.json") != []:
        pathList.append(glob.glob(f"{folderPath}/*.json"))
    return pathList

if __name__ == '__main__' :
    Key = ""
    # Parsing all input arguments, using default if not provided
    parser = argparse.ArgumentParser(description="Creating new openAI assistant")
    parser.add_argument("--name", type=str, default="Recommendation Assistant OpenAI", help="Assistant Name",required=True)
    parser.add_argument("--api_key", type=str, default=api_key, help="Openai Api key",)
    parser.add_argument("--instructions_file", type=str, default="../files/instruction.txt", help="File which contains the instruction for the assistant",)
    parser.add_argument("--model", type=str, default="gpt-4-1106-preview", help="Selected model to use",)
    parser.add_argument("--tools", type=str, default="code_interpreter,retrieval", help="Type of tools to use (code_interpreter/retrieval)",)
    parser.add_argument("--files", type=str, default="../docs/useful", help="Folder path where all files (pdf,json) are located",)
    args = parser.parse_args()
    # parser.print_help()

    try :
        Key = createAssistant(args)
        newKey="NEW_ASS_ID=" + Key
        fAss = open("include/assid.conf", "w")
        fAss.write(newKey)
        fAss.close()
        print(Key)
        os.environ['OPENAI_ASSISTANT_ID'] = Key
    except KeyError :
        print(KeyError)

    # Update assistant here to add all files in docs folder
    # fileLi = fileList("../docs")
    fileLi = fileList(args.files)
    fL = np.array(fileLi).flatten()
    for fil in fL :
        print(fil)
        # Upload a file to OpenAI
        with open(fil, 'rb') as file:
            upFile = client.files.create(file=file, purpose='assistants')
            client.beta.assistants.files.create(assistant_id=Key,file_id=upFile.id)

# Execution :
'''
    python3 assistant.py \
        --name "Issue and Recommendation assistant V_002" \
        --api_key=Add-here-the-api-key-generated-from-OpenAI-platform \
        --instructions_file=../files/instruction.txt \
        --model=gpt-4-1106-preview  \
        --tools=retrieval  \
        --files=../docs/all
'''
# python3 assistant.py --name "Issue and Recommendation assistant V_002" --instructions_file=../files/instruction.txt --model=gpt-3.5-turbo-16k --tools=retrieval --files=../docs/all