from flask import Flask, jsonify, request
from openai import OpenAI
import os
from execRequest import execReq
# from feedback import storeFeedback
from flask_cors import CORS

openAi_Api_Key = os.environ['OPENAI_API_KEY']
openAi_ass_Id = os.environ['OPENAI_ASSISTANT_ID']
api_port = int(os.environ['ASSISTANT_PORT'])
file_port = int(os.environ['FILE_PORT'])

# Define openAI assistant client 
client = OpenAI(api_key=f'{openAi_Api_Key}')


# Store received feedback in a Database or file
def storeFeedback(fdbk, recom):
    print(fdbk + recom)
    # return None
    toStore = "{\"feedback\":\"" + fdbk + "\",\"recommendations\":\"" +recom + "\"}\n"
    # f = open('../files/feedback.json', 'a+')
    f = open('/lti/share/feedback.json', 'a+')
    f.write(toStore)
    f.close()
    return fdbk + recom

app = Flask(__name__)

# To avoid CORS (Cross-origin resource sharing) Blocking
# CORS(app, origins=['http://localhost:{api_port}/api/recommendation/'])

CORS(app)

# For method get used in client end
@app.route('/api/recommendation/get', methods=['GET'])
def recommendationGet():
    apiRequest = request.args.get('apiRequest')
    if apiRequest:
        try:
            print(f"Request from : {request.remote_addr}")
            apiAssistant = client.beta.assistants.retrieve(openAi_ass_Id)
            res = execReq(client,apiRequest,apiAssistant.id)
            # print(res)
            # return jsonify(response=res)
            return res
        except ValueError:
            return jsonify(error='An error has been occured : Check your api key and/or used model')

# For method post is used in client end
@app.route('/api/recommendation/post', methods=['POST'])
def recommendationPost():
    global counter
    data = request.get_json()
    if 'apiRequest' in data:
        apiRequest = data['apiRequest']
        try:
            print(f"Request from : {request.remote_addr}")
            # add function creating client and thread for openai 
            apiAssistant = client.beta.assistants.retrieve(openAi_ass_Id)
            res = execReq(client,apiRequest,apiAssistant.id)
            # return jsonify(response=res)
            return res
        except ValueError:
            return jsonify(error='An error has been occured : Check your api key and/or used model')

# Usage for POST method
# curl -X POST -H "Content-Type: application/json" -d '{"apiRequest":"{http : [25.05, 20.4, Above], https : [36.4, 21.50, Higher], tcp : [22.01, 21.90, Normal], udp : [20.05, 20.10, Normal], icmp : [52.30, 25.03, Higher], twamp : [25.50, 22.82, Above], period : [15m, Minutes ], network:[1002,Latencetech]}"}' http://3.96.178.105:5002/api/recommendation/post
# curl -X POST -H "Content-Type: application/json" -d '{"apiRequest": {"http":[25.33, 20.3, "Above"], "https":[21.50, 21.20, "Normal"], "tcp":[40.00, 22.00, "Higher"],"udp":[22.00, 20.40, "Above"], "icmp":[32.00, 21.40, "Higher"], "twamp":[22.00, 20.00, "Normal"]}}' http://3.96.178.105:5002/api/recommendation/post
# {"apiRequest": {"http":[25.33, 20.3, "Above"], "https":[21.50, 21.20, "Normal"], "tcp":[40.00, 22.00, "Higher"],"udp":[22.00, 20.40, "Above"], "icmp":[32.00, 21.40, "Higher"], "twamp":[22.00, 20.00, "Normal"]}}
# @app.route('/my_folder/<filename>')
# def my_files(filename):
#     return send_from_directory("my_folder", filename)

# main running on port

# For method get used in client end

@app.route('/api/recommendation/feedback', methods=['GET'])
def feedbackGet():
    feedBack = request.args.get('feedB')
    recom = request.args.get('recom')
    if feedBack:
        try:
            print(f"Feedback from : {request.remote_addr}")
            # apiAssistant = client.beta.assistants.retrieve(openAi_ass_Id)
            fdbk = storeFeedback(feedBack,recom)
            return fdbk
        except ValueError:
            return jsonify(error='An error has been occured : Check your api key and/or used model')

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=api_port)